@@ -41,6 +41,7 @@ use Xacobeo::Document;
 use Xacobeo::Utils qw(:xml :dom);
 use Xacobeo::I18n;
 use Xacobeo::Timer;
+use Xacobeo::Error;
 use Xacobeo::XS qw(
 	xacobeo_populate_gtk_text_buffer
 	xacobeo_populate_gtk_tree_store
@@ -191,7 +192,7 @@ sub display_xml_node {
 	my $self = shift;
 	my ($widget_name, $node) = @_;
 	
-	my $namespaces = $self->document->namespaces;
+	my $namespaces = $self->document ? $self->document->namespaces : undef;
 	my $textview = $self->glade->get_widget($widget_name);
 
 	# It's faster to disconnect the buffer from the view and to reconnect it back
@@ -201,7 +202,13 @@ sub display_xml_node {
 
 
 	# A NodeList
-	if (isa_dom_nodelist($node)) {
+	if (! defined $node) {
+		buffer_add($buffer, error => __("Node is undef"));
+	}
+	elsif (ref($node) eq 'Xacobeo::Error') {
+		buffer_add($buffer, error => $node->message);
+	}
+	elsif (isa_dom_nodelist($node)) {
 		my @children = $node->get_nodelist;
 		my $count = scalar @children;
 
@@ -336,26 +343,61 @@ sub load_file {
 	
 	# Parse the content
 	my $t_load = Xacobeo::Timer->start(__('Load document'));
-	my $document = Xacobeo::Document->new($file);
+	my $document;
+	eval {
+		$document = Xacobeo::Document->new($file);
+	};
+	if (my $error = $@) {
+		my $message = __x("Can't read {file}: {error}", file => $file, error => $error);
+		$self->display_statusbar_message($message);
+	}
 	$self->document($document);
 	undef $t_load;
+	
+	$self->populate_widgets($file);
+	
+	$timer->stop();
+	if ($document) {
+		my $format = __n(
+			"Document loaded in %.3f second", 
+			"Document loaded in %.3f seconds", 
+			int($timer->elapsed),
+		);
+		$self->display_statusbar_message(sprintf $format, $timer->elapsed);
+	}
+	else {
+		# Invoke the time elapsed this way the value is not printed to the console
+		$timer->elapsed;
+	}
+	
+}
+
+
+#
+# Populates the different widgets after a document has been loaded
+#
+sub populate_widgets {
+	my $self = shift;
+	my ($file) = @_;
 
 	my $glade = $self->glade;
 	$glade->get_widget('window')->set_title("$APP_NAME - $file");
-
-	my $xml = $document->xml;
-	my $namespaces = $document->namespaces;
+	
+	my $document = $self->document;
+	my ($xml, $namespaces) = $document ? ($document->xml, $document->namespaces) : (undef, {});
 
 	# Update the text widget
 	my $t_syntax = Xacobeo::Timer->start(__('Syntax Highlight'));
 	$self->display_xml_node('xml-document', $xml);
 	undef $t_syntax;
+	
+	# Clear the previous results
+	$glade->get_widget('xpath-results')->get_buffer->set_text('');
 
 	# Populate the DOM view tree
 	my $t_dom = Xacobeo::Timer->start(__('DOM Tree'));
 	$self->populate_treeview($xml);
 	undef $t_dom;
-	$timer->stop();
 	
 	
 	# Populate the Namespaces view
@@ -365,18 +407,10 @@ sub load_file {
 	}
 	@{ $self->namespaces_view->{data} } = @namespaces;
 
-	my $format = __n(
-		"Document loaded in %.3f second", 
-		"Document loaded in %.3f seconds", 
-		int($timer->elapsed),
-	);
-	$self->display_statusbar_message(sprintf $format, $timer->elapsed);
-	
 	$glade->get_widget('xpath-entry')->set_sensitive(TRUE);
 }
 
 
-
 #
 # Populates the DOM tree view.
 #
@@ -388,7 +422,12 @@ sub populate_treeview {
 	my $store = $treeview->get_model;
 	
 	$treeview->set_model(undef);
-	xacobeo_populate_gtk_tree_store($store, $node, $self->document->namespaces);
+	if (defined $node and defined $store) {
+		xacobeo_populate_gtk_tree_store($store, $node, $self->document->namespaces);
+	}
+	elsif (defined $store) {
+		$store->clear();
+	}
 	$treeview->set_model($store);
 }
 
@@ -518,6 +557,10 @@ sub populate_tag_table {
 		weight     => PANGO_WEIGHT_BOLD,
 	);
 	
+	add_tag($tag_table, error =>
+		foreground => 'red',
+	);
+	
 	return $tag_table;
 }
 
@@ -568,14 +611,25 @@ sub callback_run_xpath {
 	# Run the XPath expression
 	my $xpath = $glade->get_widget('xpath-entry')->get_text;
 	my $timer = Xacobeo::Timer->start();
-	my $result = $self->document->find($xpath);
+	my $result;
+	eval {
+		$result = $self->document->find($xpath);
+	};
+	my $error = $@;
 	$timer->stop();
 	
-	my $count = isa_dom_nodelist($result) ? $result->size : 1;
-	$self->display_statusbar_message(
-		sprintf "Found %d results in %0.3f s", $count, $timer->elapsed
-	);
-	
+	if ($error) {
+		$result = Xacobeo::Error->new(xpath => $error);
+		$self->display_statusbar_message(__("XPath query issued an error"));
+	}
+	else {
+		my $count = isa_dom_nodelist($result) ? $result->size : 1;
+		my $format = __n("Found %d result in %0.3fs", "Found %d results in %0.3fs", $count);
+		$self->display_statusbar_message(
+			sprintf $format, $count, $timer->elapsed
+		);
+	}
+
 	# Display the results
 	$self->display_results($result);
 }
@@ -787,14 +841,13 @@ sub display_statusbar_message {
 sub glade_custom_handler {
 	my ($glade, $function, $name, $str1, $str2, $int1, $int2, $data) = @_;
 	my $self = $data;
-	print "Called $name->$function\n";
 	
 	my $widget;
 	if ($self->can($function)) {
 		$widget = $self->$function();
 	}
 	else {
-		my $message = "Missing method $function for creating widget $name";
+		my $message = __x("Can't create widget {name} because method {function} is missing", function => $function, name => $name);
 		warn $message;
 		$widget = Gtk2::Label->new($message);
 	}
@@ -834,6 +887,7 @@ sub create_xpath_results_view {
 	my $tag_table = populate_tag_table(Gtk2::TextTagTable->new());
 	my $buffer = Gtk2::TextBuffer->new($tag_table);
 	my $widget = Gtk2::TextView->new_with_buffer($buffer);
+	$widget->set_editable(FALSE);
 	
 	return $widget;
 }
